"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const operators_1 = require("rxjs/operators");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const socket_1 = require("./socket");
exports.book = (options) => {
    const subProps = Object.assign({ channel: 'book', symbol: options.symbol }, (options.prec && { prec: options.prec }), (options.freq && { freq: options.freq }), (options.length && { length: options.length }));
    const subscribe = () => (Object.assign({ event: 'subscribe' }, subProps));
    return socket_1.socket(subscribe);
};
exports.pair = (pair, options) => {
    const stream$ = exports.book(Object.assign({ symbol: `t${pair.split('/').join('')}` }, options));
    return stream$.pipe(operators_1.map(value => {
        if (Array.isArray(value[0])) {
            const snapshot = value;
            return snapshot.map(([price, count, amount]) => ({
                price: new bignumber_js_1.default(price),
                count: new bignumber_js_1.default(count),
                amount: new bignumber_js_1.default(amount),
            }));
        }
        const [price, count, amount] = value;
        return {
            price: new bignumber_js_1.default(price),
            count: new bignumber_js_1.default(count),
            amount: new bignumber_js_1.default(amount),
        };
    }));
};
exports.currency = (symbol, options) => {
    const stream$ = exports.book(Object.assign({ symbol: `f${symbol}` }, options));
    return stream$.pipe(operators_1.map(value => {
        if (Array.isArray(value[0])) {
            const snapshot = value;
            return snapshot.map(([rate, period, count, amount]) => ({
                rate: new bignumber_js_1.default(rate),
                period: new bignumber_js_1.default(period),
                count: new bignumber_js_1.default(count),
                amount: new bignumber_js_1.default(amount),
            }));
        }
        const [rate, period, count, amount] = value;
        return {
            rate: new bignumber_js_1.default(rate),
            period: new bignumber_js_1.default(period),
            count: new bignumber_js_1.default(count),
            amount: new bignumber_js_1.default(amount),
        };
    }));
};
const keyedSnapshot = (values) => {
    return values.reduce((carry, current) => {
        const key = (current.rate ||
            current.price).toString();
        return Object.assign({}, carry, { [key]: current });
    }, {});
};
exports.track = (initial = []) => {
    const start = keyedSnapshot(initial);
    return source => source.pipe(operators_1.scan((carry, current) => {
        if (Array.isArray(current)) {
            const snapshot = current;
            return keyedSnapshot(snapshot);
        }
        const update = current;
        const key = (update.rate ||
            update.price).toString();
        if (current.count.isGreaterThan(0)) {
            return Object.assign({}, carry, { [key]: current });
        }
        if (current.count.isZero()) {
            return R.omit([key], carry);
        }
        return carry;
    }, start), operators_1.map(value => Object.values(value)));
};
